function [Final_HRV_features,Final_SWD_features,Final_SWD_OCs] = get_features(selected_data, selected_header,leads_idx,threshold,plotting1,plotting2) %get_ECGLeads_features

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Extract features from ECG signals of every lead
% Inputs:
% 1. ECG data from available leads (data)
% 2. Header files including the number of leads (header_data)
% 3. The available leads index (in data/header file)
%
% Outputs:
% features for every ECG lead:
% 1. Age 2. Sex 3. root square mean (RSM) of the ECG leads
%
% Author: Nadi Sadr, PhD, <nadi.sadr@dbmi.emory.edu>
% Version 1.0
% Date 25-Nov-2020
% Version 2.1, 25-Jan-2021
% Version 2.2, 11-Feb-2021
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% read number of leads, sample frequency and adc_gain from the header.
[recording,Total_time,num_leads,Fs,adc_gain,age,sex,Baseline] = extract_data_from_header(selected_header);
num_leads = length(leads_idx);

Overall_filtered_signals = double.empty;
Overall_HRV_features = double.empty;
Overall_SWD_features = double.empty;
Overall_SWD_OCs = single.empty;
first_lead_index = 0;
for lead_id = 1:num_leads

A1 = selected_data(lead_id,:) == 0;
A2 = length(find(A1 == 1));

if A2 == length(selected_data(lead_id,:))
   Overall_HRV_features(:,lead_id) = Overall_HRV_features(:,lead_id-1);
   Overall_filtered_signals(:,lead_id) = zeros(size(Overall_filtered_signals,1),1);
   continue
end   
   
       LeadswGain = (selected_data(lead_id,:)-Baseline(lead_id))./adc_gain(lead_id);
       LeadswGain = single(LeadswGain);
       
       [Overall_filtered_signals(:,lead_id)] = filter_ECG(LeadswGain,Fs);
       new_Fs = 128;
       [Overall_HRV_features(:,lead_id),HRV_label] = get_HRV_features(Overall_filtered_signals(:,lead_id),new_Fs);
       nan_id = find(isnan(Overall_HRV_features(:,lead_id)) == 1);
       Overall_HRV_features(nan_id,lead_id) = 0;
       
       if HRV_label == 1
          if lead_id == 1
             first_lead_index = 1;
             Overall_HRV_features(:,lead_id) = zeros(27,1);
          else
          Overall_HRV_features(:,lead_id) = Overall_HRV_features(:,lead_id-1);
          end
       end
       
       if (lead_id == size(selected_data,1)) && (first_lead_index == 1)
          Overall_HRV_features(:,1) = Overall_HRV_features(:,2);
       end          
       
end
       summed_signal = sum(Overall_filtered_signals,2);

       [Overall_SWD_features,SWD_OCs] = get_SWD_features_2(summed_signal,new_Fs,threshold,plotting1);
              
       % Extract features from merged + High/Low merged OCs
%        [Overall_SWD_merged_features,Overall_merged_OCs] = merge_OCs(Overall_SWD_OCs,new_Fs,threshold,plotting2);
       
       % HRV Features
       stacked_features = double.empty;
       for feature_id = 1:size(Overall_HRV_features,2)
           stacked_features = [stacked_features;Overall_HRV_features(:,feature_id)];
       end
       Final_HRV_features = single([age;sex;stacked_features]);
       
       Final_SWD_features = Overall_SWD_features;%[stacked_features;Overall_SWD_merged_features];
       
       Final_SWD_OCs = SWD_OCs;
          
end
