function [Final_dataset,age,sex,Finalized_HRV,Finalized_beats,Finalized_SWD,Finalized_swarms] = get_features_modified(data,header_data,leads_idx)

data = single(data(cell2mat(leads_idx),:));

header_patient = header_data;
[~,~,~,fs,adc_gain,age,sex,Baseline] = extract_data_from_header(header_patient);

LeadswGain = double.empty;
for j = 1:size(data,1)
    LeadswGain(j,:)   = (data(j,:)-Baseline(j))./adc_gain(j);
end
data = LeadswGain;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Filtering, smoothing, baselining
num_of_leads = size(data,1);
edited_leads = single.empty;
for lead_id = 1:num_of_leads
            selected_lead = data(lead_id,:);
                                       
            % 0) Re-sample data to 500Hz
            if fs == 1000
               [p,q] = rat(500 / 1000);
               selected_lead_resampled = resample(double(selected_lead),p,q,0);
            else if fs == 257  
                    [p,q] = rat(500 / 257);
                    selected_lead_resampled = resample(double(selected_lead),p,q,0);
                 else
                    selected_lead_resampled = selected_lead;
                end
            end
            
            if (sum(selected_lead==0) == length(selected_lead))
                if lead_id == 1
                   edited_leads(:,lead_id) = single(0.5*ones(5000,1));
                else
                   edited_leads(:,lead_id) = single(edited_leads(:,lead_id-1));
                end
                
                continue
            end
                        
            % 1) Butter-worth
            fs_2 = 500;
            fs_ny = fs_2/2;
            Wp = 50/fs_ny;
            Ws = 60/fs_ny;
            [n,Wn] = buttord(Wp,Ws,1,2.5);            
            [b,a] = butter(n,Wn);
            selected_lead_butter = filtfilt(b,a,double(selected_lead_resampled));
                
            % 2) Wavelet Smoothing
            [data_waveleted,c2] = wden(selected_lead_butter,'modwtsqtwolog','s','mln',4,'db5');
            
            edited_leads(:,lead_id) = single(data_waveleted);

end
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Final_dataset = cell.empty;
% 3) Baseline
    selected_signal = edited_leads;
    yy2 = smoothdata(selected_signal,'gaussian',1000);
    data_baselined = selected_signal - yy2;
    data_baselined_normalized = normalize(data_baselined);
    Final_dataset{1,1} = data_baselined_normalized;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Selecting best 5000 samples
segment_length = 5000;
if size(Final_dataset{1,1},2) == 2
   lead_id = 1; %decider of regions in ECG
else   
   lead_id = 2; %decider of regions in ECG
end
for count = 1:length(Final_dataset)
    if (length(Final_dataset{count,1}) == segment_length) || ((length(Final_dataset{count,1}) > segment_length) && (length(Final_dataset{count,1}) < segment_length*2))
        
       Final_dataset{count,1} = Final_dataset{count,1}(1:segment_length,:);
    end
    
    if length(Final_dataset{count,1}) >= segment_length*2
        %count = 1204
        %count = 2918
        if length(Final_dataset{count,1}) > 30000
           selected_signal = Final_dataset{count,1}(1:30000,lead_id);
        else
           selected_signal = Final_dataset{count,1}(1:end,lead_id);
        end
        
       start = 1;
       movseg = 5000;
       segments = double.empty;
       for seg_id = 1:floor(length(selected_signal)./5000)
       segments(seg_id,:) = selected_signal(start:movseg,:);
       start = movseg + 1;
       movseg = movseg + 5000;
       end    
       
       RMSE = double.empty;
       for element_id = 1:size(segments,1)
              selected_segment = segments(element_id,:);
           if element_id == size(segments,1)
              next_segment =  segments(element_id-1,:);
           else
               next_segment = segments(element_id+1,:);
           end
              RMSE(element_id,1) = sqrt(mean((selected_segment - next_segment).^2));
        end
       RMSE(RMSE>2) = mean(RMSE);
       max_id = find(RMSE == max(RMSE));
       if length(max_id) > 1
          max_id = max_id(1,1);
       end
       selected_signal_all = Final_dataset{count,1}(1:end,:);
       
       if max_id == 1
          maximum_segment = selected_signal_all(1:5000,:);
       end
       
       if max_id == 2
          maximum_segment = selected_signal_all(5001:10000,:);
       end
       
       if max_id == 3
          maximum_segment = selected_signal_all(10001:15000,:);
       end
       
       if max_id == 4
          maximum_segment = selected_signal_all(15001:20000,:);
       end
       
       if max_id == 5
          maximum_segment = selected_signal_all(20001:25000,:);
       end
           
       if max_id == 6
          maximum_segment = selected_signal_all(25001:30000,:);
       end    
           
       Final_dataset{count,1} = maximum_segment;
    end
    
    if length(Final_dataset{count,1}) < segment_length
        
       Final_dataset{count,1} = [Final_dataset{count,1}(1:end,:);Final_dataset{count,1}(1:end,:)];
       
       if length(Final_dataset{count,1}) < segment_length
          Final_dataset{count,1} = [Final_dataset{count,1}(1:end,:);Final_dataset{count,1}(1:end,:)];
       end
       if length(Final_dataset{count,1}) < segment_length
          Final_dataset{count,1} = [Final_dataset{count,1}(1:end,:);Final_dataset{count,1}(1:end,:)];
       end
       if length(Final_dataset{count,1}) < segment_length
          Final_dataset{count,1} = [Final_dataset{count,1}(1:end,:);Final_dataset{count,1}(1:end,:)];
       end      
       
       Final_dataset{count,1} = Final_dataset{count,1}(1:segment_length,:);
    end
    
end

Finalized_HRV = single.empty;
Finalized_beats = single.empty;
for u = 1:length(Final_dataset)
    selected_signal = normalize(atan(Final_dataset{u,1}(:,lead_id)));
    [qrs_amp_raw,qrs_i_raw,delay] = pan_tompkin(double(selected_signal),500,0);
    if (length(qrs_i_raw) <= 2) || (isempty(qrs_i_raw) == 1)
       [qrs_amp_raw,qrs_i_raw,delay] = pan_tompkin(double(selected_signal),250,0);
    end
    if (length(qrs_i_raw) <= 2) || (isempty(qrs_i_raw) == 1)
       [qrs_amp_raw,qrs_i_raw,delay] = pan_tompkin(double(selected_signal),100,0);
    end
    if (length(qrs_i_raw) <= 2) || (isempty(qrs_i_raw) == 1)
       selected_signal = selected_signal(1000:end);
       [qrs_amp_raw,qrs_i_raw,delay] = pan_tompkin(double(selected_signal),500,0);
    end
    
    selected_portion = diff(qrs_i_raw)./500;
    selected_portion = repmat(selected_portion,1,15);
       
    selected_portion_mean = mean(selected_portion);
    selected_portion_std = std(selected_portion);
    selected_portion_variance = var(selected_portion);
    selected_portion_kurtosis = kurtosis(selected_portion);
    selected_portion_skewness = skewness(selected_portion);   
    
        [hrv_td] = hrv_time_edit(selected_portion,50/1000);
        AVNN = hrv_td.AVNN./1000;
        SDNN = hrv_td.SDNN./1000;
        RMSSD = hrv_td.RMSSD./1000;
        pNN50 = hrv_td.pNN0./100;
        SEM = hrv_td.SEM./1000;
        time_features = [AVNN;SDNN;RMSSD;pNN50;SEM];
        
        [hrv_fd] = hrv_freq_edit(selected_portion);
        hrv_fd2 = table2array(hrv_fd);
        hrv_fd2 = hrv_fd2./1000;
        freq_features = [hrv_fd2'];
        
        [hrv_nl] = hrv_nonlinear_edit(selected_portion);
        hrv_nl2 = table2array(hrv_nl);
        hrv_nl2(1,1) = hrv_nl2(1,1)./100;
        hrv_nl2(1,2) = hrv_nl2(1,2)./100;
        nonlinear_features = hrv_nl2';
        
        [hrv_frag] = hrv_fragmentation_edit(selected_portion);
        hrv_frag2 = table2array(hrv_frag);
        fragmentation_features = hrv_frag2'./100;

        all_hrv_features = [time_features;freq_features;nonlinear_features;fragmentation_features];        
        all_beats_features = [selected_portion_mean;selected_portion_std;selected_portion_variance;selected_portion_kurtosis;selected_portion_skewness];

        
        Finalized_HRV(u,:) = single(all_hrv_features);
        Finalized_beats(u,:) = single(all_beats_features);
end
[x,y] = find(isnan(Finalized_HRV) == 1);
Finalized_HRV(x,y) = 0;
[x,y] = find(isnan(Finalized_beats) == 1);
Finalized_beats(x,y) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Extracting SWD
Finalized_SWD = single.empty;
Finalized_swarms = cell.empty;
for u = 1:length(Final_dataset)
%     disp(['    ', num2str(u), '/', num2str(length(Final_dataset)), '...'])
    selected_signal = normalize(Final_dataset{u,1}(:,lead_id));
    
    x    = selected_signal;
    fs   = 500;
    dt   = 1 / fs;
    L    = length(x);
    t    = 0:dt:(L - 1) * dt;
    nfft = 2 ^ nextpow2(L);
    X    = abs(fft(x, nfft)) / L;
    f    = (fs / 2) * linspace(0, 1, nfft/2);
%     figure;
%     subplot(1, 2, 1); plot(t, x); 
%     subplot(1, 2, 2); plot(f, X(1:end/2));

    L = length(x);
    min_peak      = 0.1;
    component_std = 0.1;
    welch_window  = round(0.2*L);
    
    param_struct  = struct('P_th',         min_peak, ...
                           'StD_th',       component_std, ...
                           'Welch_window', welch_window);

%     tic
    cmps = SWD_V3(double(x), param_struct);
%     toc
    cmps(:,find(mean(cmps) == 0)) = [];
    
    if size(cmps,2) < 3
    L = length(x);
    min_peak      = 0.08;
    component_std = 0.1;
    welch_window  = round(0.2*L);
    
    param_struct  = struct('P_th',         min_peak, ...
                           'StD_th',       component_std, ...
                           'Welch_window', welch_window);

%     tic
    cmps = SWD_V3(double(x), param_struct);
%     toc
    cmps(:,find(mean(cmps) == 0)) = [];
    end
    
    if size(cmps,2) < 3
    L = length(x);
    min_peak      = 0.05;
    component_std = 0.1;
    welch_window  = round(0.2*L);
    
    param_struct  = struct('P_th',         min_peak, ...
                           'StD_th',       component_std, ...
                           'Welch_window', welch_window);

%     tic
    cmps = SWD_V3(double(x), param_struct);
%     toc
    cmps(:,find(mean(cmps) == 0)) = [];
    end
    
    if size(cmps,2) < 3
    L = length(x);
    min_peak      = 0.05;
    component_std = 0.1;
    welch_window  = round(0.2*L);
    
    param_struct  = struct('P_th',         min_peak, ...
                           'StD_th',       component_std, ...
                           'Welch_window', welch_window);

%     tic
    cmps = SWD_V3(double(x), param_struct);
%     toc
    cmps(:,find(mean(cmps) == 0)) = [];
    end
    
    
    cmps_srtd = spectal_sorting_imfs(cmps(:, 1:end));
    residual  = cmps(:, end);
    [Px, w_main] = pwelch(x);
    Pcmps   = pwelch(cmps_srtd);
    
    if size(cmps_srtd,2) < 3
       cmps_srtd(:,3) = cmps_srtd(:,2);
       Pcmps(:,3) = Pcmps(:,2);
    end
    
% % %     figure;
% % %     NoC = size(cmps_srtd, 2);
% % %     subplot(NoC+1, 2, 1); plot(x);title('Pacemaker Arrhythmia Original Signal')%ylim([min(x) max(x)])
% % %     subplot(NoC+1, 2, 2); plot(w_main/pi, Px);
% % %     for i = 1:NoC
% % % 
% % %         subplot(NoC+1, 2, 2*(i-1)+3); plot(cmps_srtd(:, i));title(['OC ', num2str(i)])%ylim([min(x) max(x)])
% % %         subplot(NoC+1, 2, 2*(i-1)+4); plot(w_main/pi, Pcmps(:, i));
% % %     end
    
    number_of_zeros = double.empty;
    for comp_id = 1:size(cmps_srtd,2)
        number_of_zeros(comp_id,1) = length(find(Pcmps(:, comp_id) < 0.1));
    end
    [~,lowest_zeros] = min(number_of_zeros);
    cmps_srtd_edit = double.empty;
    cmps_srtd_delete = cmps_srtd;
    cmps_srtd_delete(:, lowest_zeros) = [];
    cmps_srtd_edit = [cmps_srtd(:, lowest_zeros),cmps_srtd_delete];
    Pcmps_delete = Pcmps;
    Pcmps_delete(:, lowest_zeros) = [];
    Pcmps_edit = [Pcmps(:, lowest_zeros),Pcmps_delete];
   
% % %     figure;
% % %     NoC = size(cmps_srtd, 2);
% % %     subplot(NoC+1, 2, 1); plot(x);title('Pacemaker Arrhythmia Original Signal')%ylim([min(x) max(x)])
% % %     subplot(NoC+1, 2, 2); plot(w_main/pi, Px);
% % %     for i = 1:NoC
% % % 
% % %         subplot(NoC+1, 2, 2*(i-1)+3); plot(cmps_srtd_edit(:, i));title(['OC ', num2str(i)])%ylim([min(x) max(x)])
% % %         subplot(NoC+1, 2, 2*(i-1)+4); plot(w_main/pi, Pcmps_edit(:, i));
% % %     end
    
    numerofcomponents = size(cmps_srtd,2);
    
    if numerofcomponents > 3
        
    y_SWD_fix = cmps_srtd_edit;
    y_SWD_added_total_high = 0;
    y_SWD_added_total_low = 0;
    y_SWD_added_total = 0;
for SWD_count = 2:1:numerofcomponents
    [Px, w] = pwelch(y_SWD_fix(:, SWD_count));
    [max_val max_loc] = max(Px);
    [locationr locationc] = find(Px >= max_val./1.5);
    w2 = w_main./pi;
    w_final = w2(locationr);
    ffMean = mean(w_final);
         
    y_SWD_added_total = y_SWD_added_total + y_SWD_fix(:, SWD_count); 

    if ffMean > 0.02 
       y_SWD_added_total_high = y_SWD_added_total_high + y_SWD_fix(:, SWD_count); 
    end
         
    if ffMean <= 0.02
       y_SWD_added_total_low = y_SWD_added_total_low + y_SWD_fix(:, SWD_count); 
    end

end
     if length(y_SWD_added_total_high) == 1
        y_SWD_added_overall = [y_SWD_fix(:,1),y_SWD_fix(:,2),...
                          	y_SWD_added_total_low-y_SWD_fix(:,2)];
     else
     
     if length(y_SWD_added_total_low) == 1
        y_SWD_added_overall = [y_SWD_fix(:,1),y_SWD_added_total_high-y_SWD_fix(:,end),...
                          	y_SWD_fix(:,end)];
     else
     
        y_SWD_added_overall = [y_SWD_fix(:,1),y_SWD_added_total_high,...
                          	y_SWD_added_total_low];
     end
     end
%    y_SWD_added_overall = [y_SWD_fix(:,1),y_SWD_added_total,...
%                           y_SWD_fix(:, end)];
     
   y_SWD2 = y_SWD_added_overall';
   [numerofcomponents c] = size(y_SWD2);
   
    else
        
    y_SWD2 = cmps_srtd';
    
    end
    
% % %     y_SWD_plot = y_SWD2';   
% % %     figure;
% % %     Pcmps2 = pwelch(y_SWD_plot);
% % %     NoC = size(y_SWD_plot, 2);
% % %     subplot(NoC+1, 2, 1); plot(x);title('Pacemaker Arrhythmia Original Signal')%ylim([min(x) max(x)])
% % %     subplot(NoC+1, 2, 2); plot(w_main/pi, Px);
% % %     for i = 1:NoC 
% % %     subplot(NoC+1, 2, 2*(i-1)+3); plot(y_SWD_plot(:, i));title(['OC ', num2str(i)])%ylim([min(x) max(x)])
% % %     subplot(NoC+1, 2, 2*(i-1)+4); plot(w_main/pi, Pcmps2(:, i));
% % %     end

    entropy_features = double.empty;
    HFD = double.empty;
    KFD = double.empty;
    for k = 1:3
        signal = normalize(y_SWD2(k,:));
%         entropy_features(1,k) = wentropy(normalize(y_SWD2(k,:)),'shannon')./1000;
        entropy_features(1,k) = sampen(signal, 1, 0.2);
        HFD(1,k) = Higuchi_FD(signal, 5); 
        KFD(1,k) = Katz_FD(signal);
    end
    kurtosis_features = double.empty;
    for k = 1:3
        kurtosis_features(1,k) = kurtosis(normalize(y_SWD2(k,:)));
    end
    skewness_features = double.empty;
    for k = 1:3
        skewness_features(1,k) = skewness(normalize(y_SWD2(k,:)));
    end  
    
    features = [entropy_features,HFD,KFD,kurtosis_features,skewness_features];
    
    Finalized_SWD(u,:) = single(features);
    Finalized_swarms{u,1} = single(normalize(y_SWD2'));
end


end