function ObjFcn = makeObjFcn(X_train,T_train,X_test,T_test,classes_weights,classWeights)
ObjFcn = @valErrorFun;
    function [valError,cons,fileName] = valErrorFun(optVars)

inputSize = [size(X_train{1,1},1) size(X_train{1,1},2) 1];
% numHiddenUnits = 512;
numClasses = length(unique(T_train));
% 
% netWidth = 16;

layers = [
    sequenceInputLayer(inputSize,'Name','input','Normalization','zerocenter')
    sequenceFoldingLayer('Name','fold')
    
    convolution2dLayer([optVars.conv_kernel*2 2],optVars.netWidth,'stride',[4 1],'Padding','same','Name','convInp')
    batchNormalizationLayer('Name','BNInp')
    reluLayer('Name','reluInp')
    
    convolutionalUnit([optVars.conv_kernel 2],optVars.netWidth,1,'S1U1')
    additionLayer(2,'Name','add11')
    reluLayer('Name','relu11')
    convolutionalUnit([optVars.conv_kernel 2],optVars.netWidth,1,'S1U2')
    additionLayer(2,'Name','add12')
    reluLayer('Name','relu12')
    
    convolutionalUnit([optVars.conv_kernel 2],4*optVars.netWidth,[2 1],'S2U1')
    additionLayer(2,'Name','add21')
    reluLayer('Name','relu21')
    convolutionalUnit([optVars.conv_kernel 2],4*optVars.netWidth,1,'S2U2')
    additionLayer(2,'Name','add22')
    reluLayer('Name','relu22')
    
    convolutionalUnit([optVars.conv_kernel 2],8*optVars.netWidth,[2 1],'S3U1')
    additionLayer(2,'Name','add31')
    reluLayer('Name','relu31')
    convolutionalUnit([optVars.conv_kernel 2],8*optVars.netWidth,1,'S3U2')
    additionLayer(2,'Name','add32')
    reluLayer('Name','relu32')
    
    averagePooling2dLayer([optVars.conv_kernel 2],'Stride',[2 1],'Name','avgPool')

    sequenceUnfoldingLayer('Name','unfold')
    flattenLayer('Name','flatten')
    
    bilstmLayer(optVars.numHiddenUnits,'OutputMode','last','Name','bilstm')
       
    fullyConnectedLayer(numClasses,'Name','fcFinal')
    softmaxLayer('Name','softmax')
    classificationLayer('Classes',classes_weights,'ClassWeights',classWeights,'Name','classification')
%     classificationLayer('Name','classification')
    ];

lgraph = layerGraph(layers);
% figure('Units','normalized','Position',[0.2 0.2 0.6 0.6]);
% plot(lgraph);

lgraph = connectLayers(lgraph,'fold/miniBatchSize','unfold/miniBatchSize');
lgraph = connectLayers(lgraph,'reluInp','add11/in2');
lgraph = connectLayers(lgraph,'relu11','add12/in2');

% figure('Units','normalized','Position',[0.2 0.2 0.6 0.6]);
% plot(lgraph);

skip1 = [
    convolution2dLayer([1 1],4*optVars.netWidth,'Stride',[2 1],'Name','skipConv1')
    batchNormalizationLayer('Name','skipBN1')];
lgraph = addLayers(lgraph,skip1);
lgraph = connectLayers(lgraph,'relu12','skipConv1');
lgraph = connectLayers(lgraph,'skipBN1','add21/in2');

lgraph = connectLayers(lgraph,'relu21','add22/in2');

skip2 = [
    convolution2dLayer([1 1],8*optVars.netWidth,'Stride',[2 1],'Name','skipConv2')
    batchNormalizationLayer('Name','skipBN2')];
lgraph = addLayers(lgraph,skip2);
lgraph = connectLayers(lgraph,'relu22','skipConv2');
lgraph = connectLayers(lgraph,'skipBN2','add31/in2');

lgraph = connectLayers(lgraph,'relu31','add32/in2');
        

        miniBatchSize = 512;
        validationFrequency = 50;
        options = trainingOptions('adam', ...
            'InitialLearnRate',optVars.InitialLearnRate, ...
            'MaxEpochs',25, ...
            'MiniBatchSize',miniBatchSize, ...
            'L2Regularization',optVars.L2Regularization, ...
            'Shuffle','every-epoch', ...
            'Verbose',false, ...
            'Plots','training-progress', ...
            'ValidationData',{X_test,T_test}, ...
            'ValidationFrequency',validationFrequency);
        
        trainedNet = trainNetwork(X_train,T_train,lgraph,options);
        close(findall(groot,'Tag','NNET_CNN_TRAININGPLOT_UIFIGURE'))
        
        YPredicted = classify(trainedNet,X_test);
        valError = 1 - mean(YPredicted == T_test);
        
        fileName = num2str(valError) + ".mat";
%         save(fileName,'trainedNet','valError','options')
        cons = [];
        
    end
end
        