function [ mse_result, scale_axis] = mse_edit(sig, varargin)

load MSE_defaults

% Define input
p = inputParser;
p.KeepUnmatched = true;
p.addRequired('sig', @(x) isnumeric(x) && ~isscalar(x));
p.addParameter('mse_max_scale', DEFAULT_MSE_MAX_SCALE, @(x) isnumeric(x) && isscalar(x));
p.addParameter('sampen_r', DEFAULT_SAMPEN_R, @(x) isnumeric(x) && isscalar(x));
p.addParameter('sampen_m', DEFAULT_SAMPEN_M, @(x) isnumeric(x) && isscalar(x));
p.addParameter('normalize_std', DEFAULT_NORMALIZE_STD, @(x) islogical(x));
p.addParameter('plot', nargout == 0, @islogical);

% Get input
p.parse(sig);
mse_max_scale = p.Results.mse_max_scale;
sampen_r = p.Results.sampen_r;
sampen_m = p.Results.sampen_m;
normalize_std = p.Results.normalize_std;

%% MSE Calculation
% Normalize input
N = length(sig);
sig_normalized = sig - mean(sig);
if normalize_std
    sig_normalized = sig_normalized / sqrt(var(sig_normalized));
end

% Preallocate result vector
mse_result = zeros(1, mse_max_scale);

scale_axis = 1:mse_max_scale;
for scale = scale_axis
    % Split the signal into windows of length 'scale'
    max_idx = floor(N/scale) * scale;
    sig_windows = reshape(sig_normalized(1:max_idx), scale, []);
    
    % Calculate the mean of each window to obtain the 'coarse-grained' signal
    sig_coarse = mean(sig_windows, 1);
    
    % Calculate sample entropy of the coarse-grained signal
    sampen = sample_entropy(sig_coarse, sampen_m, sampen_r);

    % An infinite sample entropy is possible, since
    % SampEn = -ln(A/B) where A is the number of template matches of length
    % m+1 and B is the number of template matches of length m. It's
    % possible that A will be zero or both will be zero (it's not possible
    % that only B will be zero). In the case of A=0, we'll get infinity.
    % Set it to NaN because:
    % 1. Consistent with the case of A=B=0.
    % 2. Easier to work with nans (e.g. to exclude them from calculations).
    % We don't do this in the SampEn function because there is a different
    % meaning to both this cases which will be hidden if we do it there.
    if isinf(sampen)
        sampen = NaN;
    end

    mse_result(scale) = sampen;
end


end
