close all; clear; clc;
%% Input signal
load('A0005.mat');
x = val(1, :);

x = (x(:) - mean(x)) / std(x);  % the normalization is helpful

%% Run SWD
L = length(x);
min_peak      = 0.01;        % This parameter determines how many cmps will be extracted, the less min_peak the more components. It also detemines the execution time.
component_std = 0.01;         % This parameter must be less than 0.2. Values smaller than 0.001 makes no difference in the most cases.
welch_window  = round(0.5*L); % This parameter determines how fine or coarse will be spectrum when it calculated in the algorithm. It determines how coarse the extracted components will be.

param_struct  = struct('P_th',         min_peak, ...
                       'StD_th',       component_std, ...
                       'Welch_window', welch_window);

% With this parameter you can control the internal clustering the algorithm does. 
% If it is very small, no clustering happens. Large values (>0.4) may
% cluster all components.
param_struct.clustering_factor = 0.15;

[cmps, r] = SWD_V4(x, param_struct);

%% Sorting
cmps_srtd = cmps;
residual  = cmps(:, end);

%% Plotting results
[Px, w] = pwelch(x);
Pcmps   = pwelch(cmps_srtd);

figure; 
subplot(2, 1, 1); plot(x); title('initial signal time-domain');
subplot(2, 1, 2); plot(w/pi, Px); title('initial signal frequency-domain');

figure; 
subplot(2, 1, 1); plot(cmps_srtd); title('cmps time-domain');
subplot(2, 1, 2); plot(w/pi, Pcmps);   title('cmps frequency-domain');

figure;
NoC = size(cmps, 2)-1;
for i = 1:NoC
   
    subplot(NoC, 2, 2*(i-1)+1); plot(cmps_srtd(:, i));
    subplot(NoC, 2, 2*(i-1)+2); plot(w/pi, Pcmps(:, i));
    
end