function  model = team_training_code(input_directory,output_directory) % train_ECG_leads_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train ECG leads and obtain classifier models
% for 12-lead, 6-lead, 3-lead, 4-lead and 2-lead ECG sets
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% model: trained model
% Logistic regression models for different sets of leads
%
% Author: Erick Andres Perez Alday, PhD, <perezald@ohsu.edu>
% Version 1.0 Aug-2020
% Revision History
% By: Nadi Sadr, PhD, <nadi.sadr@dbmi.emory.edu>
% Version 2.0 1-Dec-2020
% Version 2.2 25-Jan-2021
% Version 2.3 26-April-2021
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define lead sets (e.g 12, 6, 4, 3 and 2 lead ECG sets)
twelve_leads = [{'I'}, {'II'}, {'III'}, {'aVR'}, {'aVL'}, {'aVF'}, {'V1'}, {'V2'}, {'V3'}, {'V4'}, {'V5'}, {'V6'}];
six_leads    = [{'I'}, {'II'}, {'III'}, {'aVR'}, {'aVL'}, {'aVF'}];
four_leads   = [{'I'}, {'II'}, {'III'}, {'V2'}];
three_leads  = [{'I'}, {'II'}, {'V2'}];
two_leads    = [{'I'}, {'II'}];
lead_sets = {twelve_leads, six_leads, four_leads, three_leads, two_leads};

disp('Loading data...')

% Find files.
input_files = {};
features =[];
for f = dir(input_directory)'
    if exist(fullfile(input_directory, f.name), 'file') == 2 && f.name(1) ~= '.' && all(f.name(end - 2 : end) == 'mat')
        input_files{end + 1} = f.name;
    end
end

% Extract classes from dataset.
% read number of unique classes
classes = get_classes(input_directory,input_files);
num_classes = length(classes);     % number of classes
num_files   = length(input_files);
Total_data  = cell(1,num_files);
Total_header= cell(1,num_files);

%% Load data recordings and header files
% Iterate over files.
for i = 1:num_files
    disp(['    ', num2str(i), '/', num2str(num_files), '...'])
    % Load data.
    file_tmp = strsplit(input_files{i},'.');
    tmp_input_file = fullfile(input_directory, file_tmp{1});
    [data,hea_data] = load_challenge_data(tmp_input_file);

    Total_data{i}=single(data);
    Total_header{i}=hea_data;

end

Total_data = Total_data';
Total_header = Total_header';
disp('Training model..')

%% Duplicated Diagnosis
%%%% Splitting extra diseased patients
Total_header2 = Total_header;
count = 0;
Total_header_repeated = cell.empty;
Total_data_repeated = cell.empty;
for i = 1:length(Total_header2)
    disp(['    ', num2str(i), '/', num2str(length(Total_header2)), '...'])

    header_patient = Total_header2{i,1};
    Dx = header_patient{16,1};
    Dx_splitted = strsplit(Dx,',');
    
    if size(Dx_splitted,2) > 1
        
       first_disease = Dx_splitted{1,1};
       first_header_patient = header_patient;
       first_header_patient{16,1} = first_disease;
             
       % first disease
       Total_header2{i,1} = first_header_patient;

       % extra diseases
       if size(Dx_splitted,2) >= 2
           for j = 2:size(Dx_splitted,2)
               extra_disease = str2double(Dx_splitted{1,j});
               if isnan(extra_disease) == 1
                  continue
               end
%                if (extra_disease == 426783006) %|| (extra_disease == 164934002) || (extra_disease == 39732003)
%                    continue
%                end
               count = count + 1;
               extra_header_patient = header_patient;
               letter = string(extra_disease);
               extra_header_patient{16,1} = ['#Dx: ',letter{1,1}]; 
               Total_header_repeated{count,1} = extra_header_patient;
               Total_data_repeated{count,1} = Total_data{i,1};
           end
       end  
    end
end
Total_header_fixed = [Total_header2;Total_header_repeated];
Total_data_fixed = [Total_data;Total_data_repeated];

clear Total_data
clear Total_header
clear Total_header2

%% Picking Dx alone
%%%% Dx
num_files_repeated = length(Total_header_fixed);
Total_Dx=double.empty; %cell of multiple diseases (char)
for i = 1:num_files_repeated
    disp(['    ', num2str(i), '/', num2str(num_files_repeated), '...'])
    
    header_patient = Total_header_fixed{i,1};
   
    splitted_Dx = strsplit(header_patient{16,1},' ');
    Dx = splitted_Dx{1,2};

    Total_Dx(i,1) = str2double(string(Dx));

end

%% Selecting only scored data
count = 0;
Total_data_scored = cell.empty;
Total_Dx_scored = double.empty;
header_patient = double.empty;
Total_headers_final = cell.empty;
scored_labels = [164889003;...
                 164890007;...
                 6374002;...
                 426627000;...
                 733534002;...
                 713427006;...
                 270492004;...
                 713426002;...
                 39732003;...
                 445118002;...
                 164909002;...
                 251146004;...
                 698252002;...
                 426783006;...
                 284470004;...
                 10370003;...
                 365413008;...
                 427172004;...
                 164947007;...
                 111975006;...
                 164917005;...
                 47665007;...
                 59118001;...
                 427393009;...
                 426177001;...
                 427084000;...
                 63593006;...
                 164934002;...
                 59931005;...
                 17338001];

for i = 1:num_files_repeated
    disp(['    ', num2str(i), '/', num2str(num_files_repeated), '...'])
    signal_patient = Total_data_fixed{i,1};
  
    dx_patient = Total_Dx(i,1);
    header_patient = Total_header_fixed{i,1};
    
    score_check = ismember(dx_patient,scored_labels);
    
    if score_check == 1
       count = count + 1;
       Total_data_scored{count,1} = signal_patient;
       Total_Dx_scored(count,1) = dx_patient;
       Total_headers_final{count,1} = header_patient;
    end

end
% Same labels
Total_Dx_scored_merged = Total_Dx_scored;
label_713427006 = find(Total_Dx_scored_merged == 713427006);
Total_Dx_scored_merged(label_713427006) = 59118001;
label_284470004 = find(Total_Dx_scored_merged == 284470004);
Total_Dx_scored_merged(label_284470004) = 63593006;
label_427172004 = find(Total_Dx_scored_merged == 427172004);
Total_Dx_scored_merged(label_427172004) = 17338001;
label_733534002  = find(Total_Dx_scored_merged == 733534002);
Total_Dx_scored_merged(label_733534002) = 164909002;

clear Total_data_fixed
clear Total_header_fixed

%% Finding number per label
classes = unique(Total_Dx_scored_merged);
class_X = double.empty;
for i = 1:length(classes)
    class_X(i,1) = length(find(Total_Dx_scored_merged==classes(i,1)));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Splitting per class
Data_split = cell.empty;
header_split = cell.empty;
for class_id = 1:length(classes)
    selected_class = classes(class_id,1);
    class_location = find(Total_Dx_scored_merged == selected_class);
    Data_split{class_id,1} = Total_data_scored(class_location);
    header_split{class_id,1} = Total_headers_final(class_location);
end

clear Total_data_scored
clear Total_headers_final

%% Selecting only 7500 sample per category
Data_split_reduced = cell.empty;
header_split_reduced = cell.empty;
for class_id = 1:length(classes)
    disp(['    ', num2str(class_id), '/', num2str(length(classes)), '...'])

    selected_class_number = class_X(class_id,1);
    
    if selected_class_number < 7500
       Data_split_reduced{class_id,1} = Data_split{class_id,1};
       header_split_reduced{class_id,1} = header_split{class_id,1};
    end
  
    if selected_class_number >= 7500
       all_id = [1:selected_class_number]';
       factor = round(selected_class_number/7500);
       selected_all_id = all_id(1:factor:end,1);
       Data_split_reduced{class_id,1} = Data_split{class_id,1}(selected_all_id,1);
       header_split_reduced{class_id,1}  = header_split{class_id,1}(selected_all_id,1);
    end

end

clear Data_split
clear header_split

%% Final arrangement
Final_data = cell.empty;
Final_headers = cell.empty;
Final_classes = single.empty;
for i=1:length(Data_split_reduced)
    selected_category = Data_split_reduced{i,1};
    selected_headers = header_split_reduced{i,1};
    number = length(selected_category);
    Final_data = [Final_data;selected_category];
    Final_headers = [Final_headers;selected_headers];
    Final_classes = [Final_classes;i.*ones(number,1)];
end

% % Corrupted
% corrupted_files = [9440];
% Final_data(corrupted_files) = [];
% Final_headers(corrupted_files) = [];
% Final_classes(corrupted_files) = [];

clear Data_split_reduced
clear header_split_reduced

clearvars -except Final_data Final_headers Final_classes input_directory output_directory input_files classes lead_sets

%% Load data recordings and header files
% Iterate over files.

Final_HRV_features = cell.empty;
Final_SWD_features = cell.empty;
Final_SWD_OCs = cell.empty;
for i = 1:length(Final_data)
    disp(['    ', num2str(i), '/', num2str(length(Final_data)), '...'])
    
    selected_data = Final_data{i,1};
    selected_header = Final_headers{i,1};
    
    % Check the number of available ECG leads
    tmp_hea = strsplit(selected_header{1},' ');
    num_leads = str2num(tmp_hea{2});
    [leads, leads_idx] = get_leads(selected_header,num_leads);
    threshold = 0.1;
    plotting1 = 0;
    plotting2 = 0;
    % Extract features
    tic
    [Final_HRV_features{i,1},Final_SWD_features{i,1},Final_SWD_OCs{i,1}] = get_features2(selected_data,selected_header,leads_idx,threshold,plotting1,plotting2);
    toc
    
end 

clearvars -except Final_HRV_features Final_SWD_features Final_SWD_OCs Final_data Final_headers Final_classes input_directory output_directory input_files classes lead_sets


%% Deep learning
%     
% Kf=10;
% T=ind2vec(double(Final_classes')); 
% indices = crossvalind('Kfold',Final_classes',Kf);
% 
% cp = classperf(double(Final_classes'));
% acc = double.empty;
% out_class_total = double.empty;
% T_test_total = double.empty;
% scores = cell.empty;
% out_class = double.empty;
% test_acc = double.empty;
Overall_nets = cell.empty;
Overall_nets2 = cell.empty;

for scenario_id = 1:5
scenario_id
% for i = 1:Kf
    
%     test = find(indices == i); 
%     ttrain = find(indices ~= i);
    
    X_train = Final_SWD_OCs;%(ttrain); %Features for training
    HRV_train = Final_HRV_features;%(ttrain,:);
    SWD_train = Final_SWD_features;%(ttrain,:);
    T_train = categorical(Final_classes);%(ttrain));  
for p = 1:size(X_train,1)
    X_train{p,1} = X_train{p,1}{scenario_id,1};
%     HRV_train{p,1} = HRV_train{p,1}{scenario_id,1};
    SWD_train{p,1} = SWD_train{p,1}{scenario_id,1};
end

%     X_test  = Final_SWD_OCs(test);   %Features test set
%     HRV_test = Final_HRV_features(test,:);
%     SWD_test = Final_SWD_features(test,:);
%     T_test  = categorical(Final_classes(test));
% for p = 1:size(X_test,1)
%     X_test{p,1} = X_test{p,1}{scenario_id,1};
% %     HRV_test{p,1} = HRV_test{p,1}{scenario_id,1};
%     SWD_test{p,1} = SWD_test{p,1}{scenario_id,1};
% end

classWeights = double.empty;
for weight_id = 1:length(unique(T_train))
class_count = length(find((double(T_train)==weight_id) == 1));
classWeights(weight_id,1) = (length(T_train)/class_count)./100;
end
classes_weights = ["1","2","3","4","5","6","7","8",...
                   "9","10","11","12","13","14","15","16",...
                   "17","18","19","20","21","22","23","24","25","26"];   
% classes_weights = ["1","2","3","4","5","6","7","8",...
%                    "9"];   

inputSize = [size(X_train{1,1},1) size(X_train{1,1},2) 1];
numHiddenUnits = 512;
numClasses = length(unique(T_train));

netWidth = 32;

layers = [
    sequenceInputLayer(inputSize,'Name','input','Normalization','zerocenter')
    sequenceFoldingLayer('Name','fold')
    
    convolution2dLayer([4 2],netWidth,'stride',[4 1],'Padding','same','Name','convInp')
    batchNormalizationLayer('Name','BNInp')
    reluLayer('Name','reluInp')
    
    convolutionalUnit([2 2],netWidth,1,'S1U1')
    additionLayer(2,'Name','add11')
    reluLayer('Name','relu11')
    convolutionalUnit([2 2],netWidth,1,'S1U2')
    additionLayer(2,'Name','add12')
    reluLayer('Name','relu12')
    
    convolutionalUnit([2 2],4*netWidth,[2 1],'S2U1')
    additionLayer(2,'Name','add21')
    reluLayer('Name','relu21')
    convolutionalUnit([2 2],4*netWidth,1,'S2U2')
    additionLayer(2,'Name','add22')
    reluLayer('Name','relu22')
    
    convolutionalUnit([2 2],8*netWidth,[2 1],'S3U1')
    additionLayer(2,'Name','add31')
    reluLayer('Name','relu31')
    convolutionalUnit([2 2],8*netWidth,1,'S3U2')
    additionLayer(2,'Name','add32')
    reluLayer('Name','relu32')
    
    averagePooling2dLayer([2 2],'Stride',[2 1],'Name','avgPool')

    sequenceUnfoldingLayer('Name','unfold')
    flattenLayer('Name','flatten')
    
    bilstmLayer(numHiddenUnits,'OutputMode','last','Name','bilstm')
       
    fullyConnectedLayer(numClasses,'Name','fcFinal')
    softmaxLayer('Name','softmax')
    classificationLayer('Classes',classes_weights,'ClassWeights',classWeights,'Name','classification')
%     classificationLayer('Name','classification')
    ];

lgraph = layerGraph(layers);
% figure('Units','normalized','Position',[0.2 0.2 0.6 0.6]);
% plot(lgraph);

lgraph = connectLayers(lgraph,'fold/miniBatchSize','unfold/miniBatchSize');
lgraph = connectLayers(lgraph,'reluInp','add11/in2');
lgraph = connectLayers(lgraph,'relu11','add12/in2');

% figure('Units','normalized','Position',[0.2 0.2 0.6 0.6]);
% plot(lgraph);

skip1 = [
    convolution2dLayer([1 1],4*netWidth,'Stride',[2 1],'Name','skipConv1')
    batchNormalizationLayer('Name','skipBN1')];
lgraph = addLayers(lgraph,skip1);
lgraph = connectLayers(lgraph,'relu12','skipConv1');
lgraph = connectLayers(lgraph,'skipBN1','add21/in2');

lgraph = connectLayers(lgraph,'relu21','add22/in2');

skip2 = [
    convolution2dLayer([1 1],8*netWidth,'Stride',[2 1],'Name','skipConv2')
    batchNormalizationLayer('Name','skipBN2')];
lgraph = addLayers(lgraph,skip2);
lgraph = connectLayers(lgraph,'relu22','skipConv2');
lgraph = connectLayers(lgraph,'skipBN2','add31/in2');

lgraph = connectLayers(lgraph,'relu31','add32/in2');

% figure('Units','normalized','Position',[0.2 0.2 0.6 0.6]);
% plot(lgraph)

% analyzeNetwork(lgraph)   


options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',15, ...
    'MiniBatchSize',512, ...
    'InitialLearnRate',0.001, ...
    'LearnRateSchedule','piecewise', ...
    'LearnRateDropFactor',0.1, ...
    'LearnRateDropPeriod',12, ...
    'L2Regularization',0.0001, ...
    'Shuffle','every-epoch', ...
    'Verbose',false);
% 
% , ...    
%     'ValidationData',{X_test,T_test}, ...
%     'ValidationFrequency',50, ...
%     'Plots','training-progress'

clear net
rng('default')
[net info] = trainNetwork(X_train,T_train,lgraph,options);
% [out_class,scores] = classify(net,X_test,'MiniBatchSize',256);
% test_acc(scenario_id,1) = (sum(categorical(out_class) == T_test)./numel(T_test)).*100

Overall_nets{scenario_id,1} = net;
% 
% figure;
% confusionchart(double(T_test),double(out_class), ...
%     'ColumnSummary','column-normalized', ...
%     'RowSummary','row-normalized');



%%%%%%%%%%%%%%%%%%% Activations
overall_features = single.empty;
for pat_id = 1:length(HRV_train)
    selected_HRV = HRV_train{pat_id,1};
    selected_SWD = SWD_train{pat_id,1};
    overall_features(pat_id,:) = [selected_HRV',selected_SWD'];
end

% overall_features_test = single.empty;
% for pat_id = 1:length(HRV_test)
%     selected_HRV = HRV_test{pat_id,1};
%     selected_SWD = SWD_test{pat_id,1};
%     overall_features_test(pat_id,:) = [selected_HRV',selected_SWD'];
% end


layer = 'bilstm';
featuresTrain = activations(Overall_nets{scenario_id,1},X_train,layer);
featuresTrain = single(featuresTrain');
% featuresTest = activations(Overall_nets{scenario_id,1},X_test,layer);
% featuresTest = single(featuresTest');


featuresTrain = [featuresTrain,overall_features];
% featuresTest = [featuresTest,overall_features_test];

[x,y] = find(isnan(featuresTrain) == 1);
featuresTrain(x,y) = 0;
% [x,y] = find(isnan(featuresTest) == 1);
% featuresTest(x,y) = 0;

featuresTrain = normalize(featuresTrain,2);
% featuresTest = normalize(featuresTest,2);

%%%%%%%%%%%%%% NN
numFeatures = size(featuresTrain,2);
numClasses = length(unique(T_train));
miniBatchSize = 512;
layers2 = [
    featureInputLayer(numFeatures,'Name','input')
    fullyConnectedLayer(numClasses, 'Name','fc')
    softmaxLayer('Name','sm')
    classificationLayer('Classes',classes_weights,'ClassWeights',classWeights,'Name','classification')
%     classificationLayer('Name','classification')
    ];

options = trainingOptions('adam', ...
    'ExecutionEnvironment','gpu', ...
    'MaxEpochs',15, ...
    'MiniBatchSize',miniBatchSize, ...
    'InitialLearnRate',0.001, ...
    'L2Regularization',0.0001, ...
    'Shuffle','every-epoch', ...
    'Verbose',false);

% ,...
%     'ValidationData',{featuresTest,categorical(T_test)}, ...
%     'ValidationFrequency',50, ...
%     'Plots','training-progress'

clear net2
rng(0)
[net2 info] = trainNetwork(featuresTrain,T_train,layers2,options);

Overall_nets2{scenario_id,1} = net2;

% [out_class2,scores_NN] = classify(net2,featuresTest,'MiniBatchSize',miniBatchSize);

% test_acc(scenario_id,2) = (sum(categorical(out_class2) == T_test)./numel(T_test)).*100

% figure;
% confusionchart(T_test,out_class2, ...
%     'ColumnSummary','column-normalized', ...
%     'RowSummary','row-normalized')

end


%% Summing predictions

% scores_NN = cell.empty;
% for i = 1:5
% selected_signal = X_test;
% layer = 'bilstm';
% featuresTest = activations(Overall_nets{i, 1},selected_signal,layer);
% featuresTest = single(featuresTest');
% featuresTest = [featuresTest,overall_features_test];
% [x,y] = find(isnan(featuresTest) == 1);
% featuresTest(x,y) = 0;
% featuresTest = normalize(featuresTest,2);
%        
% [~,scores_NN{i,1}] = classify(Overall_nets2{i, 1},featuresTest,'MiniBatchSize',512);
% end
% 
% scores_NN_final = single.empty;
% out_class3 = categorical;
% for pat_id = 1:size(featuresTest,1)
%    
%     selected_pred = single.empty;
% for i = 1:5
%     selected_pred(i,:) = scores_NN{i,1}(pat_id,:);
% end
% 
% scores_NN_final(pat_id,:) = normalize(sum(selected_pred),'range'); 
% [max_value,max_id] = max(scores_NN_final(pat_id,:));
% out_class3(pat_id,1) = categorical(max_id);
% 
% end
% 
% test_acc = (sum(categorical(out_class3) == T_test)./numel(T_test)).*100
% 
% figure;
% confusionchart(T_test,out_class3, ...
%     'ColumnSummary','column-normalized', ...
%     'RowSummary','row-normalized')

%% Saving models
model_12lead = [Overall_nets(1,1),Overall_nets2(1,1);...
                Overall_nets(2,1),Overall_nets2(2,1);...
                Overall_nets(3,1),Overall_nets2(3,1);...
                Overall_nets(4,1),Overall_nets2(4,1);...
                Overall_nets(5,1),Overall_nets2(5,1)];
            
model_6lead = [Overall_nets(2,1),Overall_nets2(2,1);...
               Overall_nets(5,1),Overall_nets2(5,1)];
           
model_4lead = [Overall_nets(3,1),Overall_nets2(3,1);...
               Overall_nets(4,1),Overall_nets2(4,1);...
               Overall_nets(5,1),Overall_nets2(5,1)];   
           
model_3lead = [Overall_nets(4,1),Overall_nets2(4,1);...
               Overall_nets(5,1),Overall_nets2(5,1)]; 

model_2lead = [Overall_nets(5,1),Overall_nets2(5,1)]; 

save_ECGleads_model(model_12lead,output_directory,string(classes),12);
save_ECGleads_model(model_6lead,output_directory,string(classes),6);
save_ECGleads_model(model_4lead,output_directory,string(classes),4);
save_ECGleads_model(model_3lead,output_directory,string(classes),3);
save_ECGleads_model(model_2lead,output_directory,string(classes),2);

end

function save_ECGleads_model(model,output_directory,classes,num_leads) %save_ECG_model
% Save results.
tmp_file = [num2str(num_leads),'_lead_ecg_model.mat'];
filename = fullfile(output_directory,tmp_file);
save(filename,'model','classes','-v7.3');

disp('Done.')
end

function save_ECGleads_features(features,output_directory) %save_ECG_model
% Save results.
tmp_file = 'features.mat';
filename=fullfile(output_directory,tmp_file);
save(filename,'features');
end

% find unique number of classes
function classes = get_classes(input_directory,files)

classes={};
num_files = length(files);
k=1;
for i = 1:num_files
    g = strrep(files{i},'.mat','.hea');
    input_file = fullfile(input_directory, g);
    fid=fopen(input_file);
    tline = fgetl(fid);
    tlines = cell(0,1);
    
    while ischar(tline)
        tlines{end+1,1} = tline;
        tline = fgetl(fid);
        if startsWith(tline,'#Dx')
            tmp = strsplit(tline,': ');
            tmp_c = strsplit(tmp{2},',');
            for j=1:length(tmp_c)
                idx2 = find(strcmp(classes,tmp_c{j}));
                if isempty(idx2)
                    classes{k}=tmp_c{j};
                    k=k+1;
                end
            end
            break
        end
    end
    
    fclose(fid);
    
end
classes=sort(classes);
end

function [data,tlines] = load_challenge_data(filename)

% Opening header file
fid=fopen([filename '.hea']);

if (fid<=0)
    disp(['error in opening file ' filename]);
end

tline = fgetl(fid);
tlines = cell(0,1);
while ischar(tline)
    tlines{end+1,1} = tline;
    tline = fgetl(fid);
end
fclose(fid);

f=load([filename '.mat']);

try
    data = f.val;
catch ex
    rethrow(ex);
end

end
