classdef weightedClassificationLayer < nnet.layer.ClassificationLayer
        
    properties
        % Vector of weights corresponding to the classes in the training
        % data
        ClassWeights_p
        ClassWeights_n
    end
 
    methods
        function layer = weightedClassificationLayer(classWeights_p,classWeights_n, name)
            % layer = weightedClassificationLayer(classWeights) creates a
            % weighted cross entropy loss layer. classWeights is a row
            % vector of weights corresponding to the classes in the order
            % that they appear in the training data.
            % 
            % layer = weightedClassificationLayer(classWeights, name)
            % additionally specifies the layer name.
            
            % Set class weights
            layer.ClassWeights_p = classWeights_p;
            layer.ClassWeights_n = classWeights_n;
            
            % Set layer name
            if nargin == 3
                layer.Name = name;
            end

            % Set layer description
            layer.Description = 'Weighted cross entropy';
        end

        function loss = forwardLoss(layer, Y, T)

            classWeights_p = layer.ClassWeights_p;
            classWeights_n = layer.ClassWeights_n;

            N = size(Y,2);
            Y = squeeze(Y);
            T = squeeze(T);
            
%             first_part = classWeights_p.*(T.*log(Y));
%             second_part = classWeights_n.*((1-T).*log(1-Y));
            
            loss = (-sum(classWeights_p*(T.*log(Y))+classWeights_n*((1-T).*log(1-Y))))./N;
%             loss = -(sum(classWeights_p*(T.*log(Y)))./N);
        end
        
    end
end